﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;
using Windows.Data.Json;
using Windows.Foundation.Diagnostics;
using Windows.Storage;

namespace Accordion.Model
{
    public class InspectionItem : INotifyPropertyChanged
    {


        private const string idKey = "id";
        private const string nameKey = "name";
        private const string indexKey = "index";

        private const string readOnlyKey = "readOnly";
        private const string commentKey = "comment";
        private const string commentEnabledKey = "commentEnabled";
        private const string precisionKey = "precision";
        private const string validationKey = "validation";
        private const string photoCollectionKey = "photoCollection";
        private const string availableOptionsKey = "availableOptions";

        private const string dataTypeKey = "dataType";
        private const string oldValueKey = "oldValue";
        private const string newValueKey = "newValue";

        private const string mainCategoryKey = "mainCategory";
        private const string subCategory1Key = "subCategory1";
        private const string subCategory2Key = "subCategory2";

        private const string optionKey = "option";
        private const string availableOptionListIdKey = "availableOptionListId";

        //private ObservableCollection<AvailableOption> availableOptions;
        private string availableOptionListId;

        private const string downloadsFolder = "downloads";
        private const string drawingFilePlaceholder = "[DRAWING_FILE]";
        private const string workpackInstPlaceholder = "[WORKPACK_INSTRUCTIONS]";
        private const string priorInspPlaceholder = "[PRIOR_INSPECTION]";
        private const string emptyImage = @"ms-appx:///Assets/EmptyImage.png";

        private string id;
        private string name;
        private int index;

        private bool readOnly;
        private bool commentEnabled;
        private string comment;
        private int precision;
        private Validation validation;

        private string dataType;
        private string oldValue;
        private string newValue;

        private string mainCategory;
        private string subCategory1;
        private string subCategory2;

        private bool isValid;
        private string validationErrorMessage;

        // Define templates if needed for grid in InspectionItemGridControlView.xaml.cs
        public const string DATA_TYPE_BOOLEAN = "boolean";
        public const string DATA_TYPE_DATE = "date";
        public const string DATA_TYPE_PASTDATE = "pastDate";
        public const string DATA_TYPE_DECIMAL = "decimal";
        public const string DATA_TYPE_DROPDOWN = "dropDown";
        public const string DATA_TYPE_RADIOBUTTON = "radioButtons";
        public const string DATA_TYPE_STRING = "string";
        public const string DATA_TYPE_LONGSTRING = "longstring";
        public const string DATA_TYPE_DOCLINK = "docLink";
        public const string DATA_TYPE_RESETDOCLINK = "resetDocLink";  // resetable doc link
        public const string DATA_TYPE_MULTISELECT = "multiSelect";

        public event PropertyChangedEventHandler PropertyChanged;

        // Private - won't serialize
        private InspectionComponent parent;

        public InspectionItem(InspectionComponent parentComp) : this()
        {
            // Clones need parents, but I don't want to make a public setter
            parent = parentComp;
        }
        public InspectionItem()
        {
            id = string.Empty;
            name = string.Empty;
            index = 0;

            readOnly = false;
            comment = null;
            precision = 0;
            validation = new Validation();
            //Needs to be nullable
            //PhotoCollection = new ObservableCollection<InspectionPhoto>();

            dataType = string.Empty;
            oldValue = string.Empty;
            newValue = string.Empty;

            mainCategory = string.Empty;
            subCategory1 = string.Empty;
            subCategory2 = string.Empty;

            // Stop keeping this copy and just reference the component list
            //availableOptions = new ObservableCollection<AvailableOption>();
            availableOptionListId = string.Empty;
        }




        // Clone method for creating new feature/points
        public InspectionItem clone()
        {
            InspectionItem clone = new InspectionItem(this.parent)
            {
                Id = this.id,
                Name = this.name,
                DataType = this.dataType,
                Validation = this.validation,
                MainCategory = this.mainCategory,
                SubCategory1 = this.subCategory1,
                SubCategory2 = this.subCategory2,
                ReadOnly = this.readOnly,
                AvailableOptionListId = this.availableOptionListId,
                // Don't copy comment, but copy enabled flag
                CommentEnabled = this.commentEnabled
                //Comment = (comment == null ? null : "")
                // ignoring photo collection 

            };

            //if (dataType.Equals(DATA_TYPE_DROPDOWN))
            //{
            //    foreach (AvailableOption opt in availableOptions)
            //    {
            //        clone.availableOptions.Add(opt);
            //    }
            //}
            return clone;
        }




        //TODO make this its own class
        private void Validate(string value)
        {
            bool originalState = IsValid;

            IsValid = true;
            ValidationErrorMessage = string.Empty;

            // By default, readonly items should be valid && Checkboxes are never null
            if (readOnly || dataType.Equals(DATA_TYPE_BOOLEAN))
                return;


            if (Validation.Required && string.IsNullOrEmpty(value))
            {
                IsValid = false;
                ValidationErrorMessage = "Field is Required";
            }

            if (Validation.MaxLength.HasValue)
            {
                if (value.Length > Validation.MaxLength)
                {
                    IsValid = false;
                    ValidationErrorMessage = "Maximum Characters Exceeded";
                }
            }

            if (dataType.Equals(DATA_TYPE_DECIMAL))
            {
                // If not required and missing that's ok
                if (!Validation.Required && string.IsNullOrEmpty(value))
                {
                    IsValid = true;
                    return;
                }


                try
                {
                    decimal testValue;
                    if (Decimal.TryParse(value, out testValue))
                    {
                        if (IsValid != originalState)
                            OnValidStateChangedEvent(new ValidStateChangeEventArgs() { IncrementCount = IsValid });
                        return;
                    }
                }
                catch (Exception) { }
                IsValid = false;
                ValidationErrorMessage = "Numeric field required";
            }
            if (IsValid != originalState)
                OnValidStateChangedEvent(new ValidStateChangeEventArgs() { IncrementCount = IsValid });
        }



        //setDateElement(inspectionItemJsonObject, oldValueKey, OldValue);
        private void setDateElement(JsonObject jObject, string valueKey, object value)
        {
            if (String.IsNullOrEmpty((string)value))
                return;

            try
            {
                DateTime dt = DateTime.Parse((string)value);
                jObject.SetNamedValue(valueKey, JsonValue.CreateStringValue(dt.ToString("MM/dd/yyyy")));

            }
            catch (Exception)
            { }
        }

        #region Properties

        public string Id
        {
            get
            {
                return id;
            }
            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("value");
                }
                id = value;
            }
        }

        public string Name
        {
            get
            {
                return name;
            }
            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("value");
                }
                if (value != this.Name)
                {
                    name = value;
                    NotifyPropertyChanged();
                }

            }
        }

        public int Index
        {
            get
            {
                return index;
            }
            set
            {
                index = value;
            }

        }

        public bool IsEnabled
        {
            get
            {
                return !ReadOnly;
            }
        }

        public bool ReadOnly
        {
            get
            {
                return readOnly;
            }
            set
            {
                readOnly = value;
            }
        }

        public bool CommentEnabled
        {
            get { return commentEnabled; }
            set
            {
                if (value != commentEnabled)
                {
                    commentEnabled = value;
                    NotifyPropertyChanged();
                }
            }
        }

        public string Comment
        {
            get
            {
                return comment;
            }
            set
            {
                // if (value != comment && !string.IsNullOrEmpty(value))
                if (value != comment)
                {
                    comment = value;
                    NotifyPropertyChanged();
                }
            }

        }

        public int Precision
        {
            get
            {
                return precision;
            }
            set
            {
                precision = value;
            }
        }

        public Validation Validation
        {
            get
            {
                return validation;
            }
            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("value");
                }
                validation = value;
            }
        }


        public string DataType
        {
            get
            {
                return dataType;
            }
            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("value");
                }
                dataType = value;
            }
        }

        public string OldValue
        {
            get
            {
                return oldValue;
            }
            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("value");
                }
                oldValue = value;
            }
        }

        public string NewValue
        {
            get
            {
                return newValue;
            }
            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("value");
                }
                Validate(value);
                newValue = value;
            }
        }

        public string MainCategory
        {
            get
            {
                return mainCategory;
            }
            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("value");
                }
                mainCategory = value;
            }
        }

        public string SubCategory1
        {
            get
            {
                return subCategory1;
            }
            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("value");
                }
                subCategory1 = value;
            }
        }

        public string SubCategory2
        {
            get
            {
                return subCategory2;
            }
            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("value");
                }
                subCategory2 = value;
            }
        }

        public string SubCategory
        {
            get
            {
                // MikeJ prefers not to show the SubCat1 each time
                if (!string.IsNullOrEmpty(SubCategory2))
                    return SubCategory2;
                else if (!string.IsNullOrEmpty(SubCategory1))
                    return SubCategory1;
                else
                    return string.Empty;

                /*
                if (!string.IsNullOrEmpty(SubCategory1) && !string.IsNullOrEmpty(SubCategory2))
                {
                    return string.Format("{0} - {1}", SubCategory1, SubCategory2);
                }
                else if (string.IsNullOrEmpty(SubCategory1))
                {
                    return string.Format("{0}", SubCategory1);
                }
                else if (string.IsNullOrEmpty(SubCategory2))
                {
                    return string.Format("Unknown - {0}", SubCategory2);
                }
                else
                {
                    return string.Empty;
                }
                */
            }
        }

        public ObservableCollection<AvailableOption> AvailableOptions
        {
            get
            {
                if (!String.IsNullOrEmpty(availableOptionListId))
                    return parent.AvailableOptionCollections[availableOptionListId].AvailableOptions;
                else
                    // This really shouldn't happen, but protect in case.
                    return new ObservableCollection<AvailableOption>();
            }
        }

        public string AvailableOptionListId
        {
            get
            {
                return availableOptionListId;
            }
            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("value");
                }
                availableOptionListId = value;
            }
        }

        public bool IsValid
        {
            get
            {
                return isValid;
            }
            set
            {
                if (value != this.isValid)
                {
                    this.isValid = value;
                    NotifyPropertyChanged();
                }
            }
        }
        public string ValidationErrorMessage
        {
            get
            {
                return validationErrorMessage;
            }
            set
            {
                if (value != this.validationErrorMessage)
                {
                    validationErrorMessage = value;
                    NotifyPropertyChanged();
                }
            }
        }

        #endregion

        private void NotifyPropertyChanged([CallerMemberName] string propertyName = "")
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        //===================== set up event for anytime the IsValid state changes
        // Define event to support the valid count headers
        public event EventHandler<ValidStateChangeEventArgs> ValidStateChangedEvent;

        protected virtual void OnValidStateChangedEvent(ValidStateChangeEventArgs eventArgs)
        {
            EventHandler<ValidStateChangeEventArgs> handler = ValidStateChangedEvent;
            if (handler != null)
            {
                handler(this, eventArgs);
            }
        }
    }

    public class ValidStateChangeEventArgs : EventArgs
    {
        public bool IncrementCount { get; set; }
    }
}
